@@ -1,4 +1,4 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -8,91 +8,94 @@ use strict;
 
 package Log::Report;
 use vars '$VERSION';
-$VERSION = '0.997';
+$VERSION = '1.00';
 
 use base 'Exporter';
 
-use List::Util qw/first/;
+use List::Util         qw/first/;
+use Scalar::Util       qw/blessed/;
 
-# domain 'log-report' via work-arounds:
-#     Log::Report cannot do "use Log::Report"
+use Log::Report::Util;
 
+### if you change anything here, you also have to change Log::Report::Minimal
 my @make_msg         = qw/__ __x __n __nx __xn N__ N__n N__w/;
-my @functions        = qw/report dispatcher try/;
+my @functions        = qw/report dispatcher try textdomain/;
 my @reason_functions = qw/trace assert info notice warning
    mistake error fault alert failure panic/;
 
 our @EXPORT_OK = (@make_msg, @functions, @reason_functions);
 
-require Log::Report::Util;
-require Log::Report::Message;
-require Log::Report::Dispatcher;
-require Log::Report::Dispatcher::Try;
-
-# See section Run modes
-my %is_reason = map {($_=>1)} @Log::Report::Util::reasons;
-my %is_fatal  = map {($_=>1)} qw/ERROR FAULT FAILURE PANIC/;
-my %use_errno = map {($_=>1)} qw/FAULT ALERT FAILURE/;
-
-sub _whats_needed(); sub dispatcher($@);
+sub _whats_needed(); sub dispatcher($@); sub textdomain(@);
 sub trace(@); sub assert(@); sub info(@); sub notice(@); sub warning(@);
 sub mistake(@); sub error(@); sub fault(@); sub alert(@); sub failure(@);
 sub panic(@);
 sub __($); sub __x($@); sub __n($$$@); sub __nx($$$@); sub __xn($$$@);
 sub N__($); sub N__n($$); sub N__w(@);
 
-require Log::Report::Translator::POT;
-
-my $reporter;
-my %domain_start;
-my %settings;
-my $default_mode = 0;
-
 #
 # Some initiations
 #
 
-__PACKAGE__->_setting('log-report', translator =>
-    Log::Report::Translator::POT->new(charset => 'utf-8'));
+my $reporter     = {};
+my $default_mode = 0;
+my @nested_tries;
 
-__PACKAGE__->_setting('rescue', translator => Log::Report::Translator->new);
+# we can only load these after Log::Report has compiled, because
+# the use this module themselves.
+
+require Log::Report::Die;
+require Log::Report::Domain;
+require Log::Report::Message;
+require Log::Report::Exception;
+require Log::Report::Dispatcher;
+require Log::Report::Dispatcher::Try;
+
+#eval "require Log::Report::Translator::POT"; panic $@ if $@;
+#, translator => Log::Report::Translator::POT->new(charset => 'utf-8');
+textdomain 'log-report';
 
 dispatcher PERL => 'default', accept => 'NOTICE-';
 
 
-# $^S = $EXCEPTIONS_BEING_CAUGHT; parse: undef, eval: 1, else 0
-
 sub report($@)
-{   my $opts   = ref $_[0] eq 'HASH' ? +{ %{ (shift) } } : {};
+{   my $opts = ref $_[0] eq 'HASH' ? +{ %{ (shift) } } : {};
     my $reason = shift;
-    my $stop = exists $opts->{is_fatal} ? $opts->{is_fatal} :$is_fatal{$reason};
+    my $stop = exists $opts->{is_fatal} ? $opts->{is_fatal} : is_fatal $reason;
+
+    my $try  = $nested_tries[-1];
+    my @disp = ($stop && $try) ? () : @{$reporter->{needs}{$reason} || []};
+    push @disp, $try if defined $try && $try->needs($reason);
 
     # return when no-one needs it: skip unused trace() fast!
-    my $disp = $reporter->{needs}{$reason};
-    $disp || $stop or return;
+    @disp || $stop or return;
 
-    $is_reason{$reason}
+    is_reason $reason
         or error __x"token '{token}' not recognized as reason", token=>$reason;
 
     $opts->{errno} ||= $!+0 || $? || 1
-        if $use_errno{$reason} && !defined $opts->{errno};
+        if use_errno($reason) && !defined $opts->{errno};
 
     if(my $to = delete $opts->{to})
     {   # explicit destination, still disp may not need it.
         if(ref $to eq 'ARRAY')
-        {   my %disp = map {$_->name => $_} @$disp;
-            $disp    = [ grep defined, @disp{@$to} ];
+        {   my %disp = map +($_->name => $_), @disp;
+            @disp    = grep defined, @disp{@$to};
         }
         else
-        {   $disp    = [ grep $_->name eq $to, @$disp ];
+        {   @disp    = grep $_->name eq $to, @disp;
         }
-        @$disp || $stop
+        @disp || $stop
             or return;
     }
 
+    my $message = shift;
+
+    unless(Log::Report::Dispatcher->can('collectLocation'))
+    {   # internal Log::Report error can result in "deep recursions".
+        eval "require Carp"; Carp::confess($message);
+    }
     $opts->{location} ||= Log::Report::Dispatcher->collectLocation;
 
-    my $message = shift;
     my $exception;
     if(UNIVERSAL::isa($message, 'Log::Report::Message'))
     {   @_==0 or error __x"a message object is reported with more parameters";
@@ -108,32 +111,33 @@ sub report($@)
     }
 
     if(my $to = $message->to)
-    {   $disp    = [ grep $_->name eq $to, @$disp ];
-        @$disp or return;
+    {   @disp = grep $_->name eq $to, @disp;
+        @disp or return;
     }
 
-    my @last_call;     # call Perl dispatcher always last
-    if($reporter->{filters})
+    my @last_call;  # call Perl dispatcher always last, it calls real die
+    my $domain = $message->domain;
+    if(my $filters = $reporter->{filters})
     {
       DISPATCHER:
-        foreach my $d (@$disp)
+        foreach my $d (@disp)
         {   my ($r, $m) = ($reason, $message);
-            foreach my $filter ( @{$reporter->{filters}} )
+            foreach my $filter (@$filters)
             {   next if keys %{$filter->[1]} && !$filter->[1]{$d->name};
-                ($r, $m) = $filter->[0]->($d, $opts, $r, $m);
+                ($r, $m) = $filter->[0]->($d, $opts, $r, $m, $domain);
                 $r or next DISPATCHER;
             }
 
             if($d->isa('Log::Report::Dispatcher::Perl'))
-                 { @last_call = ($d, { %$opts }, $r, $m) }
-            else { $d->log($opts, $r, $m) }
+                 { @last_call = ($d, { %$opts }, $r, $m, $domain) }
+            else { $d->log($opts, $r, $m, $domain) }
         }
     }
     else
-    {   foreach my $d (@$disp)
+    {   foreach my $d (@disp)
         {   if($d->isa('Log::Report::Dispatcher::Perl'))
-                 { @last_call = ($d, { %$opts }, $reason, $message) }
-            else { $d->log($opts, $reason, $message) }
+                 { @last_call = ($d, { %$opts }, $reason, $message, $domain) }
+            else { $d->log($opts, $reason, $message, $domain) }
         }
     }
 
@@ -143,8 +147,8 @@ sub report($@)
     }
 
     if($stop)
-    {   # ^S = EXCEPTIONS_BEING_CAUGHT, within eval or try
-        $^S or exit($opts->{errno} || 0);
+    {   # $^S = $EXCEPTIONS_BEING_CAUGHT; parse: undef, eval: 1, else 0
+        (defined($^S) ? $^S : 1) or exit($opts->{errno} || 0);
 
         $! = $opts->{errno} || 0;
         $@ = $exception || Log::Report::Exception->new(report_opts => $opts
@@ -152,7 +156,7 @@ sub report($@)
         die;   # $@->PROPAGATE() will be called, some eval will catch this
     }
 
-    @$disp;
+    @disp;
 }
 
 
@@ -173,12 +177,14 @@ sub dispatcher($@)
     if($command eq 'list')
     {   mistake __"the 'list' sub-command doesn't expect additional parameters"
            if @_;
-        return values %{$reporter->{dispatchers}};
+        my @disp = values %{$reporter->{dispatchers}};
+        push @disp, $nested_tries[-1] if @nested_tries;
+        return @disp;
     }
     if($command eq 'needs')
     {   my $reason = shift || 'undef';
         error __"the 'needs' sub-command parameter '{reason}' is not a reason"
-            unless $is_reason{$reason};
+            unless is_reason $reason;
         my $disp = $reporter->{needs}{$reason};
         return $disp ? @$disp : ();
     }
@@ -242,26 +248,26 @@ sub try(&@)
       and report {location => [caller 0]}, PANIC =>
           __x"odd length parameter list for try(): forgot the terminating ';'?";
 
-    local $reporter->{dispatchers} = undef;
-    local $reporter->{needs};
-
-    my $disp = dispatcher TRY => 'try', @_;
+    my $disp = Log::Report::Dispatcher::Try->new(TRY => 'try', @_);
+    push @nested_tries, $disp;
 
     my ($ret, @ret);
     if(!defined wantarray)  { eval { $code->() } } # VOID   context
     elsif(wantarray) { @ret = eval { $code->() } } # LIST   context
     else             { $ret = eval { $code->() } } # SCALAR context
 
-    my $err = $@;
-    if(   $err
-       && !$disp->wasFatal
-       && !UNIVERSAL::isa($err, 'Log::Report::Exception'))
-    {   eval "require Log::Report::Die"; panic $@ if $@;
-        ($err, my($opts, $reason, $text)) = Log::Report::Die::die_decode($err);
+    my $err  = $@;
+    pop @nested_tries;
+
+    my $is_exception = blessed $err && $err->isa('Log::Report::Exception');
+    if($err && !$is_exception && !$disp->wasFatal)
+    {   ($err, my($opts, $reason, $text)) = Log::Report::Die::die_decode($err);
         $disp->log($opts, $reason, __$text);
     }
 
-    $disp->died($err);
+    $disp->died($err)
+        if $err && ($is_exception ? $err->isFatal : 1);
+
     $@ = $disp;
 
     wantarray ? @ret : $ret;
@@ -281,12 +287,7 @@ sub failure(@) {report FAILURE => @_}
 sub panic(@)   {report PANIC   => @_}
 
 
-sub _default_domain(@)
-{   my $f = $domain_start{$_[1]} or return undef;
-    my $domain;
-    do { $domain = $_->[1] if $_->[0] < $_[2] } for @$f;
-    $domain;
-}
+sub _default_domain(@) { pkg2domain $_[0] }
 
 sub __($)
 {   Log::Report::Message->new
@@ -361,24 +362,19 @@ sub N__w(@) {split " ", $_[0]}
 sub import(@)
 {   my $class = shift;
 
+    my $to_level   = ($_[0] && $_[0] =~ m/^\+\d+$/ ? shift : undef) || 0;
     my $textdomain = @_%2 ? shift : undef;
-    my %opts   = @_;
-    my $syntax = delete $opts{syntax} || 'SHORT';
-    my ($pkg, $fn, $linenr) = caller;
+    my %opts       = @_;
 
-    if(my $trans = delete $opts{translator})
-    {   $class->translator($textdomain, $trans, $pkg, $fn, $linenr);
+    my ($pkg, $fn, $linenr) = caller $to_level;
+    my $domain;
+
+    if(defined $textdomain)
+    {   pkg2domain $pkg, $textdomain, $fn, $linenr;
+        $domain = textdomain $textdomain;
     }
 
-    if(my $native = delete $opts{native_language})
-    {   my ($lang) = parse_locale $native;
-
-        error "the specified native_language '{locale}' is not a valid locale"
-          , locale => $native unless defined $lang;
-
-        $class->_setting($textdomain, native_language => $native
-          , $pkg, $fn, $linenr);
-    }
+    ### Log::Report options
 
     if(exists $opts{mode})
     {   $default_mode = delete $opts{mode} || 0;
@@ -386,75 +382,71 @@ sub import(@)
         dispatcher mode => $default_mode, 'ALL';
     }
 
-    push @{$domain_start{$fn}}, [$linenr => $textdomain];
-
-    my @export = (@functions, @make_msg);
-
-    if($syntax eq 'SHORT')
-    {   push @export, @reason_functions
+    my @export;
+    if(my $in = delete $opts{import})
+    {   push @export, ref $in eq 'ARRAY' ? @$in : $in;
     }
-    elsif($syntax ne 'REPORT' && $syntax ne 'LONG')
-    {   error __x"syntax flag must be either SHORT or REPORT, not `{syntax}'"
-          , syntax => $syntax;
+    else
+    {   push @export, @functions, @make_msg;
+
+        my $syntax = delete $opts{syntax} || 'SHORT';
+        if($syntax eq 'SHORT')
+        {   push @export, @reason_functions
+        }
+        elsif($syntax ne 'REPORT' && $syntax ne 'LONG')
+        {   error __x"syntax flag must be either SHORT or REPORT, not `{flag}' in {fn} line {line}"
+              , flag => $syntax, fn => $fn, line => $linenr;
+        }
     }
 
-    $class->export_to_level(1, undef, @export);
+    $class->export_to_level(1+$to_level, undef, @export);
+
+    ### Log::Report::Domain configuration
+
+    if(!%opts) { }
+    elsif($domain)
+    {   $domain->configure(%opts, where => [$pkg, $fn, $linenr ]) }
+    else
+    {   error __x"no domain for configuration options in {fn} line {line}"
+          , fn => $fn, line => $linenr;
+    }
 }
 
-
+# deprecated, since we have a ::Domain object in 1.00
 sub translator($;$$$$)
-{   my ($class, $domain) = (shift, shift);
+{   # replaced by (textdomain $domain)->configure
 
-    @_ or return $class->_setting($domain => 'translator')
-              || $class->_setting(rescue  => 'translator');
+    my ($class, $name) = (shift, shift);
+    my $domain = textdomain $name
+        or error __x"textdomain `{domain}' for translator not defined"
+             , domain => $name;
 
-    defined $domain
-        or error __"textdomain for translator not defined";
+    @_ or return $domain->translator;
 
     my ($translator, $pkg, $fn, $line) = @_;
     ($pkg, $fn, $line) = caller    # direct call, not via import
         unless defined $pkg;
 
     $translator->isa('Log::Report::Translator')
-        or error __"translator must be a Log::Report::Translator object";
+        or error __x"translator must be a {pkg} object for {domain}"
+              , pkg => 'Log::Report::Translator', domain => $name;
 
-    $class->_setting($domain, translator => $translator, $pkg, $fn, $line);
-}
-
-# c_method setting TEXTDOMAIN, NAME, [VALUE]
-# When a VALUE is provided (of unknown structure) then it is stored for the
-# NAME related to TEXTDOMAIN.  Otherwise, the value related to the NAME is
-# returned.  The VALUEs may only be set once in your program, and count for
-# all packages in the same TEXTDOMAIN.
-
-sub _setting($$;$)
-{   my ($class, $domain, $name, $value) = splice @_, 0, 4;
-    $domain ||= 'rescue';
-
-    defined $value
-        or return ($settings{$domain} ? $settings{$domain}{$name} : undef);
-
-    # Where is the setting done?
-    my ($pkg, $fn, $line) = @_;
-    ($pkg, $fn, $line) = caller    # direct call, not via import
-         unless defined $pkg;
-
-    my $s = $settings{$domain} ||= {_pkg => $pkg, _fn => $fn, _line => $line};
-
-    error __x"only one package can contain configuration; for {domain} already in {pkg} in file {fn} line {line}"
-        , domain => $domain, pkg => $s->{_pkg}
-        , fn => $s->{_fn}, line => $s->{_line}
-           if $s->{_pkg} ne $pkg || $s->{_fn} ne $fn;
-
-    error __x"value for {name} specified twice", name => $name
-        if exists $s->{$name};
-
-    $s->{$name} = $value;
+    $domain->configure(translator => $translator, where => [$pkg, $fn, $line]);
 }
 
 
-sub isValidReason($) { $is_reason{$_[1]} }
-sub isFatal($)       { $is_fatal{$_[1]} }
+sub textdomain(@)
+{   # used for testing
+    return delete $reporter->{textdomains}{$_[0]}
+        if @_==2 && $_[1] eq 'DELETE';
+
+    my $name   = (@_%2 ? shift : _default_domain(caller)) || 'default';
+    my $domain = $reporter->{textdomains}{$name}
+        ||= Log::Report::Domain->new(name => $name);
+
+    $domain->configure(@_, where => [caller]) if @_;
+    $domain;
+}
 
 
 sub needs(@)
