@@ -1,4 +1,4 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -7,7 +7,7 @@ use strict;
 
 package Log::Report::Dispatcher::Callback;
 use vars '$VERSION';
-$VERSION = '0.997';
+$VERSION = '1.00';
 
 use base 'Log::Report::Dispatcher';
 
@@ -28,7 +28,7 @@ sub init($)
 sub callback() {shift->{callback}}
 
 
-sub log($$$)
+sub log($$$$)
 {   my $self = shift;
     $self->{callback}->($self, @_);
 }
