@@ -1,4 +1,4 @@
-# Copyrights 2007-2013 by [Mark Overmeer].
+# Copyrights 2007-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -7,7 +7,7 @@ use strict;
 
 package Log::Report::Dispatcher::Try;
 use vars '$VERSION';
-$VERSION = '0.997';
+$VERSION = '1.00';
 
 use base 'Log::Report::Dispatcher';
 
@@ -24,7 +24,7 @@ sub init($)
 {   my ($self, $args) = @_;
     defined $self->SUPER::init($args) or return;
     $self->{exceptions} = delete $args->{exceptions} || [];
-    $self->{died} = delete $args->{died};
+    $self->{died}       = delete $args->{died};
     $self;
 }
 
@@ -45,20 +45,23 @@ sub died(;$)
 sub exceptions() { @{shift->{exceptions}} }
 
 
-sub log($$$)
-{   my ($self, $opts, $reason, $message) = @_;
+sub log($$$$)
+{   my ($self, $opts, $reason, $message, $domain) = @_;
 
     # If "try" does not want a stack, because of its mode,
     # then don't produce one later!  (too late)
     $opts->{stack}    ||= [];
     $opts->{location} ||= '';
 
-    push @{$self->{exceptions}}
-      , Log::Report::Exception->new
-          ( reason      => $reason
-          , report_opts => $opts
-          , message     => $message
-          );
+    my $e = Log::Report::Exception->new
+     ( reason      => $reason
+     , report_opts => $opts
+     , message     => $message
+     );
+
+    my $is_fatal = exists $opts->{is_fatal} ? $opts->{is_fatal} : $e->isFatal;
+
+    push @{$self->{exceptions}}, $e;
 
     # later changed into nice message
     $self->{died} ||= $opts->{is_fatal};
